@@ -5,7 +5,7 @@ use warnings;
 use Wasm::Wasmtime::FFI;
 
 # ABSTRACT: Global configuration for Wasm::Wasmtime::Engine
-our $VERSION = '0.06'; # VERSION
+our $VERSION = '0.07'; # VERSION
 
 
 $ffi_prefix = 'wasm_config_';
@@ -29,6 +29,25 @@ foreach my $prop (qw( debug_info wasm_threads wasm_reference_types
 }
 
 
+foreach my $prop (qw( static_memory_maximum_size static_memory_guard_size dynamic_memory_guard_size ))
+{
+  my $f = eval { $ffi->function( "wasmtime_config_${prop}_set" => [ 'wasm_config_t', 'uint64' ] => 'void' => sub {
+    my($xsub, $self, $value) = @_;
+    $xsub->($self, $value);
+    $self;
+  }) };
+  if($f)
+  {
+    $f->attach($prop);
+  }
+  else
+  {
+    no strict 'refs';
+    *$prop = sub { Carp::croak("property $prop is not available") };
+  }
+}
+
+
 my %strategy = (
   auto      => 0,
   cranelift => 1,
@@ -149,7 +168,7 @@ Wasm::Wasmtime::Config - Global configuration for Wasm::Wasmtime::Engine
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -218,10 +237,30 @@ L<https://github.com/webassembly/bulk-memory>
 
 =head2 wasm_multi_value
 
+ $config->wasm_multi_value($bool)
+
 Configures whether the wasm multi value proposal is enabled.
 
 L<https://github.com/webassembly/multi-value>
 
+=head2 static_memory_maximum_size
+
+ $config->static_memory_maximum_size($size);
+
+Configure the static memory maximum size.
+
+=head2 static_memory_guard_size
+
+ $config->static_memory_guard_size($size);
+
+Configure the static memory guard size.
+
+=head2 dynamic_memory_guard_size
+
+ $config->dynamic_memory_guard_size($size);
+
+Configure the dynamic memory guard size.
+
 =head2 strategy
 
  $config->strategy($strategy);
