@@ -6,7 +6,7 @@ use Wasm::Wasmtime::FFI;
 use Wasm::Wasmtime::Config;
 
 # ABSTRACT: Wasmtime engine class
-our $VERSION = '0.06'; # VERSION
+our $VERSION = '0.07'; # VERSION
 
 
 $ffi_prefix = 'wasm_engine_';
@@ -16,6 +16,13 @@ $ffi->load_custom_type('::PtrObject' => 'wasm_engine_t' => __PACKAGE__ );
 $ffi->attach( [ 'new_with_config' => 'new' ] => ['wasm_config_t'] => 'wasm_engine_t' => sub {
   my($xsub, $class, $config) = @_;
   $config ||= Wasm::Wasmtime::Config->new;
+  if(defined $ENV{PERL_WASM_WASMTIME_MEMORY})
+  {
+    my($static_memory_maximum_size, $static_memory_guard_size, $dynamic_memory_guard_size) = split /:/, $ENV{PERL_WASM_WASMTIME_MEMORY};
+    $config->static_memory_maximum_size($static_memory_maximum_size);
+    $config->static_memory_guard_size($static_memory_guard_size);
+    $config->dynamic_memory_guard_size($dynamic_memory_guard_size);
+  }
   my $self = $xsub->($config),
   delete $config->{ptr};
   $self;
@@ -37,7 +44,7 @@ Wasm::Wasmtime::Engine - Wasmtime engine class
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
