@@ -2,24 +2,25 @@ package Wasm::Wasmtime::ExportType;
 
 use strict;
 use warnings;
+use Ref::Util qw( is_blessed_ref );
 use Wasm::Wasmtime::FFI;
 use Wasm::Wasmtime::ExternType;
 
 # ABSTRACT: Wasmtime export type class
-our $VERSION = '0.06'; # VERSION
+our $VERSION = '0.07'; # VERSION
 
 
 $ffi_prefix = 'wasm_exporttype_';
 $ffi->load_custom_type('::PtrObject' => 'wasm_exporttype_t' => __PACKAGE__);
 
 
-$ffi->attach( new => ['wasm_byte_vec_t*', 'wasm_externtype_t'] => 'wasm_exporttype_t' => sub {
+$ffi->attach( new => ['wasm_byte_vec_t*', 'opaque'] => 'wasm_exporttype_t' => sub {
   my $xsub = shift;
   my $class = shift;
-  if(defined $_[1] && ref($_[1]) eq 'Wasm::Wasmtime::ExternType')
+
+  # not sure this is actually useful...
+  if(defined $_[1] && is_blessed_ref $_[1] && $_[1]->isa('Wasm::Wasmtime::ExternType'))
   {
-    # not sure this is actually useful?
-    # doesn't seem to bee a way to new an wasm_externtype_t
     my $name = Wasm::Wasmtime::ByteVec->new(shift);
     my $externtype = shift;
     my $self = $xsub->($name, $externtype->{ptr});
@@ -68,7 +69,7 @@ Wasm::Wasmtime::ExportType - Wasmtime export type class
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -84,7 +85,8 @@ version 0.06
    )
  });
  
- my($foo, $bar) = $module->exports;
+ 
+ my($foo, $bar) = @{ $module->exports };
  
  print $foo->name, "\n";        # foo
  print $foo->type->kind, "\n";  # func
