@@ -2,7 +2,8 @@ package Wasm::Wasmtime::FFI;
 
 use strict;
 use warnings;
-use FFI::Platypus 1.00;
+use FFI::C 0.05;
+use FFI::Platypus 1.26;
 use FFI::Platypus::Buffer ();
 use FFI::CheckLib 0.26 qw( find_lib );
 use Sub::Install;
@@ -10,7 +11,7 @@ use Devel::GlobalDestruction ();
 use base qw( Exporter );
 
 # ABSTRACT: Private class for Wasm::Wasmtime
-our $VERSION = '0.06'; # VERSION
+our $VERSION = '0.07'; # VERSION
 
 
 our @EXPORT = qw( $ffi $ffi_prefix _generate_vec_class _generate_destroy );
@@ -22,6 +23,7 @@ sub _lib
 
 our $ffi_prefix = 'wasm_';
 our $ffi = FFI::Platypus->new( api => 1 );
+FFI::C->ffi($ffi);
 $ffi->lib(__PACKAGE__->_lib);
 $ffi->mangler(sub {
   my $name = shift;
@@ -187,96 +189,61 @@ if($ffi->find_symbol('wasmtime_error_message'))
   });
 }
 
-{ package Wasm::Wasmtime::CBC;
+{ package Wasm::Wasmtime::Val::Of;
+  FFI::C->union(of_t => [
+    i32     => 'sint32',
+    i64     => 'sint64',
+    f32     => 'float',
+    f64     => 'double',
+    anyref  => 'opaque',
+    funcref => 'opaque',
+  ]);
+}
 
-  use Convert::Binary::C;
-  use base qw( Exporter );
+my %kind = (
+  0   => 'i32',
+  1   => 'i64',
+  2   => 'f32',
+  3   => 'f64',
+  128 => 'anyref',
+  129 => 'funcref',
+);
 
-  our @EXPORT_OK = qw( perl_to_wasm wasm_to_perl wasm_allocate wasm_type wasm_memcpy );
+{ package Wasm::Wasmtime::Val;
+  FFI::C->struct(wasm_val_t => [
+    kind => 'uint8',
+    of   => 'of_t',
+  ]);
 
-  $INC{'Wasm/Wasmtime/CBC.pm'} = __FILE__;
+  sub to_perl
+  {
+    my $self = shift;
+    my $kind = $kind{$self->kind};
+    $self->of->$kind;
+  }
+}
 
-  # CBC is probably not how we want to do this long term, but atm
-  # Platypus does not support Unions or arrays of records so.
-  our $cbc = Convert::Binary::C->new(
-    Alignment => 8,
-    LongSize => 8, # CBC does not apparently use the native alignment by default *sigh*
-  );
-  $cbc->parse(q{
-    typedef struct wasm_val_t {
-      unsigned char kind;
-      union {
-        signed int i32;
-        signed long i64;
-        float f32;
-        double f64;
-        void *anyref;
-        void *funcref;
-      } of;
-    } wasm_val_t;
-    typedef wasm_val_t wasm_val_vec_t[];
+{ package Wasm::Wasmtime::ValVec;
+  FFI::C->array(wasm_val_vec_t => [
+    'wasm_val_t',
+  ], { nullable => 1 });
+
+  sub to_perl
+  {
+    my $self = shift;
+    map { $_->to_perl } @$self
+  }
+
+  $ffi->attach_cast('from_c', 'opaque', 'wasm_val_vec_t', sub {
+    my($xsub, undef, $ptr) = @_;
+    $xsub->($ptr);
   });
 
-  sub perl_to_wasm
+  sub from_perl
   {
-    my $vals = shift;
-    my $types = shift;
-    $cbc->pack('wasm_val_vec_t', [map {
-      {
-        kind => $_->kind_num,
-        of => {
-          $_->kind => shift @$vals,
-        }
-      }
-    } @$types]);
+    my($class, $vals, $types) = @_;
+    @$vals ? $class->new([map { { kind => $_->kind_num, of => { $_->kind => shift @$vals } } } @$types]) : undef;
   }
-
-  my %kind = (
-    0   => 'i32',
-    1   => 'i64',
-    2   => 'f32',
-    3   => 'f64',
-    128 => 'anyref',
-    129 => 'funcref',
-  );
-
-  sub wasm_to_perl
-  {
-    my $vals = shift;
-    map {
-      $_->{of}->{$kind{$_->{kind}}};
-    } @{ $cbc->unpack('wasm_val_vec_t', $vals) };
-  }
-
-  my $size = $cbc->sizeof('wasm_val_t');
-
-  sub wasm_allocate
-  {
-    my $count = shift;
-    $count ? "\0" x ($count * $size) : undef;
-  }
-
-  sub wasm_type
-  {
-    my $count = shift || 0;
-    my $name = "x_wasm_" . $count;
-    eval { $ffi->type($name) };
-    if($@)
-    {
-      $ffi->type(
-        $count > 0 ? 'string(' . ($count * $size) . ')*' : 'opaque',
-        $name,
-      );
-    }
-    $name;
-  }
-
-  my $ffi2 = FFI::Platypus->new( api => 1, lib => [undef] );
-  $ffi2->attach( [ memcpy => 'wasm_memcpy' ] => ['opaque','string','size_t'] => 'opaque' => sub {
-    my $xsub = shift;
-    $xsub->($_[0], $_[1], length $_[1]);
-  });
-
 }
 
 1;
@@ -293,7 +260,7 @@ Wasm::Wasmtime::FFI - Private class for Wasm::Wasmtime
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -316,6 +283,16 @@ This is a private class used internally by L<Wasm::Wasmtime> classes.
 
 =back
 
+=head1 SEE ALSO
+
+=over 4
+
+=item L<Wasm>
+
+=item L<Wasm::Wasmtime>
+
+=back
+
 =head1 AUTHOR
 
 Graham Ollis <plicease@cpan.org>
