@@ -2,12 +2,15 @@ package Wasm::Wasmtime::FuncType;
 
 use strict;
 use warnings;
+use base qw( Wasm::Wasmtime::ExternType );
 use Ref::Util qw( is_arrayref );
 use Wasm::Wasmtime::FFI;
 use Wasm::Wasmtime::ValType;
+use constant is_functype => 1;
+use constant kind => 'functype';
 
 # ABSTRACT: Wasmtime function type class
-our $VERSION = '0.06'; # VERSION
+our $VERSION = '0.07'; # VERSION
 
 
 $ffi_prefix = 'wasm_functype_';
@@ -47,15 +50,7 @@ $ffi->attach( results => ['wasm_functype_t'] => 'wasm_valtype_vec_t*' => sub {
   $xsub->($self)->to_list;
 });
 
-
-# actually returns a wasm_externtype_t, but recursion
-$ffi->attach( as_externtype => ['wasm_functype_t'] => 'opaque' => sub {
-  my($xsub, $self) = @_;
-  require Wasm::Wasmtime::ExternType;
-  my $ptr = $xsub->($self);
-  Wasm::Wasmtime::ExternType->new($ptr, $self->{owner} || $self);
-});
-
+__PACKAGE__->_cast(0);
 _generate_destroy();
 
 1;
@@ -72,7 +67,7 @@ Wasm::Wasmtime::FuncType - Wasmtime function type class
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -116,12 +111,6 @@ Returns a list of the parameter types for the function type, as L<Wasm::Wasmtime
 
 Returns a list of the result types for the function type, as L<Wasm::Wasmtime::ValType> objects.
 
-=head2 as_externtype
-
- my $externtype = $functype->as_externtype
-
-Returns the L<Wasm::Wasmtime::ExternType> for this function type.
-
 =head1 SEE ALSO
 
 =over 4
