@@ -2,29 +2,39 @@ package Wasm::Wasmtime::ImportType;
 
 use strict;
 use warnings;
+use Carp ();
+use Ref::Util qw( is_blessed_ref );
 use Wasm::Wasmtime::FFI;
 use Wasm::Wasmtime::ExternType;
 
 # ABSTRACT: Wasmtime import type class
-our $VERSION = '0.06'; # VERSION
+our $VERSION = '0.07'; # VERSION
 
 
 $ffi_prefix = 'wasm_importtype_';
 $ffi->load_custom_type('::PtrObject' => 'wasm_importtype_t' => __PACKAGE__);
 
 
-$ffi->attach( new => ['wasm_byte_vec_t*', 'wasm_externtype_t'] => 'wasm_importtype_t' => sub {
+$ffi->attach( new => ['wasm_byte_vec_t*', 'wasm_byte_vec_t*', 'opaque'] => 'wasm_importtype_t' => sub {
   my $xsub = shift;
   my $class = shift;
-  if(defined $_[2] && ref($_[2]) eq 'Wasm::Wasmtime::ExternType')
+  if(defined $_[2] && is_blessed_ref $_[2])
   {
-    my $module = Wasm::Wasmtime::ByteVec->new(shift);
-    my $name = Wasm::Wasmtime::ByteVec->new(shift);
-    my $externtype = shift;
-    my $self = $xsub->($module, $name, $externtype);
-    $module->delete;
-    $name->delete;
-    $self;
+    my $externtype = $_[2];
+    # not sure this is actually useful?
+    if(is_blessed_ref($externtype) && $externtype->isa('Wasm::Wasmtime::ExternType'))
+    {
+      my $module = Wasm::Wasmtime::ByteVec->new(shift);
+      my $name = Wasm::Wasmtime::ByteVec->new(shift);
+      my $self = $xsub->($module, $name, $externtype->{ptr});
+      $module->delete;
+      $name->delete;
+      return $self;
+    }
+    else
+    {
+      Carp::croak("Not an externtype");
+    }
   }
   else
   {
@@ -75,7 +85,7 @@ Wasm::Wasmtime::ImportType - Wasmtime import type class
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -87,11 +97,11 @@ version 0.06
    )
  });
  
- my($hello) = $module->imports;
+ my $hello = $module->imports->[0];
  
  print $hello->module, "\n";     # xx
  print $hello->name, "\n";       # hello
- print $hello->type->kind, "\n"; # func
+ print $hello->type->kind, "\n"; # functype
 
 =head1 DESCRIPTION
 
@@ -106,7 +116,7 @@ signature and other configuration details for import objects.
  my $importtype = Wasm::Wasmtime::ImportType->new(
    $module,       # Wasm::Wasmtime::Module
    $name,         # string
-   $externtype,   # Wasm::Wasmtime::ExternType
+   $externtype,   # Wasm::Wasmtime::FuncType, ::MemoryType, ::GlobalType or ::TableType
  );
 
 Creates a new import type object.
