@@ -2,12 +2,15 @@ package Wasm::Wasmtime::TableType;
 
 use strict;
 use warnings;
+use base qw( Wasm::Wasmtime::ExternType );
 use Wasm::Wasmtime::FFI;
 use Wasm::Wasmtime::ValType;
 use Ref::Util qw( is_ref is_plain_arrayref );
+use constant is_tabletype => 1;
+use constant kind => 'tabletype';
 
 # ABSTRACT: Wasmtime table type class
-our $VERSION = '0.06'; # VERSION
+our $VERSION = '0.07'; # VERSION
 
 
 $ffi_prefix = 'wasm_tabletype_';
@@ -59,15 +62,7 @@ $ffi->attach( limits => ['wasm_tabletype_t'] => 'uint32[2]' => sub {
   $xsub->($self);
 });
 
-
-# actually returns a wasm_externtype_t, but recursion
-$ffi->attach( as_externtype => ['wasm_tabletype_t'] => 'opaque' => sub {
-  my($xsub, $self) = @_;
-  require Wasm::Wasmtime::ExternType;
-  my $ptr = $xsub->($self);
-  Wasm::Wasmtime::ExternType->new($ptr, $self->{owner} || $self);
-});
-
+__PACKAGE__->_cast(2);
 _generate_destroy();
 
 1;
@@ -84,7 +79,7 @@ Wasm::Wasmtime::TableType - Wasmtime table type class
 
 =head1 VERSION
 
-version 0.06
+version 0.07
 
 =head1 SYNOPSIS
 
@@ -125,12 +120,6 @@ Returns the L<Wasm::Wasmtime::ValType> for this table type.
 
 Returns the limits as an array reference.
 
-=head2 as_externtype
-
- my $externtype = $tabletype->as_externtype
-
-Returns the L<Wasm::Wasmtime::ExternType> for this table type.
-
 =head1 SEE ALSO
 
 =over 4
